import java.util.*;

public class GrapheListe implements Graphe{

    // Les structures pour coder le graphe
    private Node[] nodes;
    private LinkedList[] adjacents;

    // les variables globales
    private int temps; // pour PP
    private int[] stats = new int[2]; // 0 pour les acces aux Noeuds et Arcs; 1 pour les tests "simples"

    public GrapheListe(int n){
	nodes = new Node[n];
	adjacents = new LinkedList[n];
	for (int i=0; i<n; i++) {
	    nodes[i] = new Node(i);
	    adjacents[i] = new LinkedList();
	}
    }

    private void resetStats() {
	for (int i=0; i<stats.length; i++)
	    stats[i] = 0;
    }

    private void incStatAcces(int i) {
	stats[0] += i;
    }

    private void incStatTests(int i) {
	stats[1] += i;
    }

    public int[] getStats() {
	return stats;
    }

    public int getNbNoeuds() {
	return nodes.length;
    }

    public int getNbArcs() {
	int total = 0;
	for (int i=0; i<adjacents.length; i++)
	    total += adjacents[i].size();
	return total;
    }

    private boolean addArc(int s, int t) {
	if (!adjacents[s].contains(nodes[t])){
	    adjacents[s].add(nodes[t]);
	    return true;
	}
	else
	    return false;
    }
    
    public boolean addArc(int ind, int s, int t) {
	// l'indice de l'arc ne sert  rien
	return addArc(s,t);
    }
    
    public Node getNode(int i) {
	return nodes[i];
    }

    public void afficheSuccesseurs(Node s) {
	resetStats();
	for (Iterator it = adjacents[s.getIdent()].iterator(); it.hasNext();) {
	    incStatAcces(1);
	    Node u = (Node)it.next();
	    System.out.print(u);
	}
	System.out.println();
    }
    
    public void affichePredecesseurs(Node t) {
	resetStats();
	for (int i=0; i<adjacents.length; i++)
	    for (Iterator it = adjacents[i].iterator(); it.hasNext();) {
		incStatAcces(1);
		Node u = (Node)it.next();
		incStatTests(1);
		if (u == t)
		    System.out.print(nodes[i]);
	    }
	System.out.println();
    }

    public boolean possedeBoucle() {
	resetStats();
	for (int i=0; i<adjacents.length; i++)
	    for (Iterator it = adjacents[i].iterator(); it.hasNext();) {
		incStatAcces(1);
		Node u = (Node)it.next();
		incStatTests(1);
		if (u == nodes[i])
		    return true;
	    }
	return false;
    }
    
    public void parcoursLargeur(Node s){
	resetStats();
	for (int i=0; i<nodes.length; i++){
	    incStatAcces(1);
	    nodes[i].setCouleur(0); // blanc;
	    nodes[i].setPere(null);
	    nodes[i].setDistance(Integer.MAX_VALUE);
	}
	s.setCouleur(1); // gris
	s.setDistance(0); // sommet de dpart
	LinkedList fifo = new LinkedList();
	fifo.add(s);
	while (!fifo.isEmpty()){
	    incStatAcces(1);
	    Node u = (Node)fifo.getFirst();
	    for (Iterator it=adjacents[u.getIdent()].iterator();it.hasNext();){
		incStatAcces(1);
		Node v = (Node)it.next();
		incStatTests(1);
		if (v.getCouleur() == 0){
		    v.setPere(u);
		    v.setCouleur(1);
		    v.setDistance(u.getDistance()+1);
		    fifo.add(v);
		}
	    }
	    u.setCouleur(2); // noir
	    fifo.removeFirst();
	}
    }

    public void parcoursProfondeur() {
	resetStats();
	temps = 0;
	for (int i=0; i<nodes.length; i++){
	    incStatAcces(1);
	    nodes[i].setCouleur(0); // blanc;
	    nodes[i].setPere(null);
	}
	for (int i=0; i<nodes.length; i++) {
	    incStatAcces(1);
	    incStatTests(1);
	    if (nodes[i].getCouleur() == 0)
		visiter_PP(nodes[i]);
	}
    }

    private void visiter_PP(Node s) {
	//System.out.println(s);
	temps++;
	s.setDebut(temps);
	s.setCouleur(1); // gris
	for (Iterator it = adjacents[s.getIdent()].iterator(); it.hasNext();){
	    incStatAcces(1);
	    Node t = (Node)it.next();
	    incStatTests(1);
	    if (t.getCouleur()==0){
		t.setPere(s);
		visiter_PP(t);
	    }
	}
	temps++;
	s.setFin(temps);
	s.setCouleur(2); // noir
    }

}
