import java.util.*;

public class GrapheMatriceAdj implements Graphe{

    // Les structures pour coder le graphe
    private Node[] nodes;
    private int[][] adjacence;

    // les variables globales
    private int temps; // pour PP
    private int[] stats = new int[2]; // 0 pour les acces aux Noeuds et Arcs; 1 pour les tests "simples"

    public GrapheMatriceAdj(int n){
	nodes = new Node[n];
	adjacence = new int[n][n];
	for (int i=0; i<n; i++) {
	    nodes[i] = new Node(i);
	    for (int j=0; j<n; j++) {
		adjacence[i][j] = 0;
	    }
	}
    }

    private void resetStats() {
	for (int i=0; i<stats.length; i++)
	    stats[i] = 0;
    }

    private void incStatAcces(int i) {
	stats[0] += i;
    }

    private void incStatTests(int i) {
	stats[1] += i;
    }

    public int[] getStats() {
	return stats;
    }

    public int getNbNoeuds() {
	return nodes.length;
    }

    public int getNbArcs() {
	int total = 0;
	for (int i=0; i<nodes.length; i++)
	    for (int j=0; j<nodes.length; j++)
		if (adjacence[i][j] == 1)
		    total++;
	return total;
    }
    
    private boolean addArc(int s, int t) {
	if (adjacence[s][t] != 1){
	    adjacence[s][t] = 1;
	    return true;
	}
	else
	    return false;
    }

    public boolean addArc(int ind, int s, int t) {
	// l'indice de l'arc ne sert  rien
	return addArc(s,t);
    }

    public Node getNode(int i) {
	return nodes[i];
    }

    public void afficheSuccesseurs(Node s) {
	resetStats();
	int i = s.getIdent();
	for (int j=0; j<nodes.length; j++) {
	    incStatTests(1);
	    if (adjacence[i][j] == 1) {
		incStatAcces(1);
		System.out.print(nodes[j]);
	    }
	}
	System.out.println();
    }
    
    public void affichePredecesseurs(Node t) {
	resetStats();
	int j = t.getIdent();
	for (int i=0; i<nodes.length; i++) {
	    incStatTests(1);
	    if (adjacence[i][j] == 1) {
		incStatAcces(1);
		System.out.print(nodes[i]);
	    }
	}
	System.out.println();
    }

    public boolean possedeBoucle() {
	resetStats();
	for (int i=0; i<nodes.length; i++) {
	    incStatTests(1);
	    if (adjacence[i][i] == 1) {
		return true;
	    }
	}
	return false;
    }
    
    public void parcoursLargeur(Node s){
	resetStats();
	for (int i=0; i<nodes.length; i++){
	    incStatAcces(1);
	    nodes[i].setCouleur(0); // blanc;
	    nodes[i].setPere(null);
	    nodes[i].setDistance(Integer.MAX_VALUE);
	}
	s.setCouleur(1); // gris
	s.setDistance(0); // sommet de dpart
	LinkedList fifo = new LinkedList();
	fifo.add(s);
	while (!fifo.isEmpty()){
	    incStatAcces(1);
	    Node u = (Node)fifo.getFirst();
	    int i = u.getIdent();
	    for (int j=0; j<nodes.length; j++) {
		incStatTests(1);
		if (adjacence[i][j] == 1) {
		    incStatAcces(1);
		    Node v = getNode(j);
		    incStatTests(1);
		    if (v.getCouleur() == 0){
			v.setPere(u);
			v.setCouleur(1);
			v.setDistance(u.getDistance()+1);
			fifo.add(v);
		    }
		}
	    }
	    u.setCouleur(2); // noir
	    fifo.removeFirst();
	}
    }
    
    public void parcoursProfondeur() {
	resetStats();
	temps = 0;
	for (int i=0; i<nodes.length; i++){
	    incStatAcces(1);
	    nodes[i].setCouleur(0); // blanc;
	    nodes[i].setPere(null);
	}
	for (int i=0; i<nodes.length; i++) {
	    incStatAcces(1);
	    incStatTests(1);
	    if (nodes[i].getCouleur() == 0)
		visiter_PP(nodes[i]);
	}
    }

    private void visiter_PP(Node s) {
	//System.out.println(s);
	temps++;
	s.setDebut(temps);
	s.setCouleur(1); // gris
	int i = s.getIdent();
	for (int j=0; j<nodes.length; j++) {
	    incStatTests(1);
	    if (adjacence[i][j] == 1) {
		incStatAcces(1);
		Node t = getNode(j);
		incStatTests(1);
		if (t.getCouleur()==0){
		    t.setPere(s);
		    visiter_PP(t);
		}
	    }
	}
	temps++;
	s.setFin(temps);
	s.setCouleur(2); // noir
    }

}
