import java.io.*;
import java.text.*;
import java.util.*;

public class Test{

    private static void usage(){
	System.out.println("Usage : Test nbGraphes nbMaxSommets");
    }
    public static void main(String[] args) throws Exception {
	if (args.length !=2)
	    usage();
	else {
	    int nbTests = Integer.parseInt(args[0]);
	    int nbMaxSommets = Integer.parseInt(args[1]);
	    // les differentes implementation d'un meme graphe
	    Graphe[] gr = new Graphe[3];
	    // pour chaque implementation, 
	    // les statistiques pour les 5 algorithmes compars
	    int[][][] stats = new int[gr.length][5][2];
	    // pour fabriquer aleatoirement les graphes
	    Random alea = new Random();
	    // pour sauvegarder les statistiques
	    FileWriter res = new FileWriter("graphesComplexites.csv");
	    res.write(",NbSommets,NbArcs,Structure,"+
		      "AffichageSuccesseurs,,"+
		      "AffichagePredecesseurs,,"+
		      "PossedeBoucle,,"+
		      "ParcoursLargeur,,"+
		      "ParcoursProfondeur,\n");
	    res.write(",,,,"+
		      "NbAcces,NbTests,"+
		      "NbAcces,NbTests,"+
		      "NbAcces,NbTests,"+
		      "NbAcces,NbTests,"+
		      "NbAcces,NbTests\n");
	    res.write("\n");
	    // une serie de graphes differents
	    for (int num=0; num<nbTests; num++){
		// Creation et sauvegarde du graphe au format dot
		System.out.println("\tCreation du graphe numero : "+num);
		FileWriter dot = new FileWriter("graphe"+num+".dot");
		dot.write("digraph \"Graphe"+num+"\" {\n"+
			  "\tnode[shape=\"box\"]\n");
		// dimentionnement du graphe (sommets, arcs)
		int nbSommets = alea.nextInt(nbMaxSommets)+1;
		for (int i=0; i<nbSommets; i++)
		    dot.write("\ts"+i+"[]\n");
		int nbArcs = alea.nextInt(nbSommets * nbSommets +1);
		gr[0] = new GrapheListe(nbSommets);
		gr[1] = new GrapheMatriceAdj(nbSommets);
		gr[2] = new GrapheMatriceInc(nbSommets, nbArcs);
		
		// creation aleatoire des arcs
		for (int i=0; i<nbArcs; i++){
		    int n = alea.nextInt(nbSommets * nbSommets);
		    if (gr[0].addArc(i,n/nbSommets, n%nbSommets)) {
			gr[1].addArc(i,n/nbSommets, n%nbSommets);
			gr[2].addArc(i,n/nbSommets, n%nbSommets);
			dot.write("\ts"+n/nbSommets+
				  " -> s"+n%nbSommets+"[]\n");
		    }
		}
		dot.write("}\n");
		dot.close();		
		// Comparaison des 5 algorithmes
		System.out.println("\tCalcule des complexites du graphe numero : "+num);
		int s = alea.nextInt(nbSommets);
		for (int g=0; g<gr.length; g++) {
		    gr[g].afficheSuccesseurs(gr[g].getNode(s));
 		    stats[g][0][0] = gr[g].getStats()[0];
 		    stats[g][0][1] = gr[g].getStats()[1];
		    gr[g].affichePredecesseurs(gr[g].getNode(s));
 		    stats[g][1][0] = gr[g].getStats()[0];
 		    stats[g][1][1] = gr[g].getStats()[1];
		    gr[g].possedeBoucle();
 		    stats[g][2][0] = gr[g].getStats()[0];
 		    stats[g][2][1] = gr[g].getStats()[1];
		    gr[g].parcoursLargeur(gr[g].getNode(s));
 		    stats[g][3][0] = gr[g].getStats()[0];
 		    stats[g][3][1] = gr[g].getStats()[1];
		    gr[g].parcoursProfondeur();
 		    stats[g][4][0] = gr[g].getStats()[0];
 		    stats[g][4][1] = gr[g].getStats()[1];
		}
		// sauvegarde des resultats
		res.write("Graphe "+num+","+nbSommets+","+nbArcs);
		for (int g=0; g<gr.length; g++) {
		    if (g==0)
			res.write(",Liste");
		    else if (g==1)
			res.write(",,,Matrice adjacence");
		    else if (g==2)
			res.write(",,,Matrice incidence");
		    for (int algo=0; algo<stats[g].length; algo++)
			for (int c=0; c<stats[g][algo].length; c++)
			    res.write(","+stats[g][algo][c]);
		    res.write("\n");
		    res.flush();
		}
		res.write("\n");
	    }
	    res.close();
	}
    }
}
